From 1d9514137bcea2f21ea1354851dfa3ea04904e0f Mon Sep 17 00:00:00 2001
From: Timo Sirainen <timo.sirainen@dovecot.fi>
Date: Fri, 7 Sep 2018 20:04:48 +0300
Subject: [PATCH] doveadm: Make sure to stop running commands requiring synced
 ring when ring is no longer synced

A command could desync the ring, and no more commands should be run
afterwards.

This also fixes a bug where a command that desyncs the ring is supposed to
wait for the ring being synced before returning OK, but returns it
immediately instead.
---
 src/director/doveadm-connection.c | 6 ++++--
 1 file changed, 4 insertions(+), 2 deletions(-)

diff --git a/src/director/doveadm-connection.c b/src/director/doveadm-connection.c
index be2c03f2f0..953fcc2b48 100644
--- a/src/director/doveadm-connection.c
+++ b/src/director/doveadm-connection.c
@@ -1171,7 +1171,8 @@ static void doveadm_connections_continue_reset_cmds(void)
 
 void doveadm_connections_ring_synced(void)
 {
-	while (doveadm_ring_sync_pending_connections != NULL) {
+	while (doveadm_ring_sync_pending_connections != NULL &&
+	       conn->dir->ring_synced) {
 		struct doveadm_connection *conn =
 			doveadm_ring_sync_pending_connections;
 		doveadm_connection_ring_sync_callback_t *callback =
@@ -1183,5 +1184,6 @@ void doveadm_connections_ring_synced(void)
 		io_set_pending(conn->io);
 		callback(conn);
 	}
-	doveadm_connections_continue_reset_cmds();
+	if (conn->dir->ring_synced)
+		doveadm_connections_continue_reset_cmds();
 }
-- 
2.18.0

